"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Any = void 0;
exports.Array = ArrayJsonSchema;
exports.Boolean = exports.BigInt = void 0;
exports.Eq = EqJsonSchema;
exports.Integer = exports.Inline = void 0;
exports.Intersect = IntersectJsonSchema;
exports.Number = exports.Null = exports.Never = void 0;
exports.Object = ObjectJsonSchema;
exports.Optional = OptionalJsonSchema;
exports.Record = RecordJsonSchema;
exports.Symbol = exports.String = void 0;
exports.Tuple = TupleJsonSchema;
exports.Undefined = void 0;
exports.Union = UnionJsonSchema;
exports.Void = exports.Unknown = void 0;
var _schemaCore = /*#__PURE__*/require("@traversable/schema-core");
var _registry = /*#__PURE__*/require("@traversable/registry");
var _specification = /*#__PURE__*/require("./specification.js");
/** @internal */
const Object_assign = globalThis.Object.assign;
/** @internal */
const Object_keys = globalThis.Object.keys;
const SymbolJsonSchema = exports.Symbol = /*#__PURE__*/Object_assign(_schemaCore.t.symbol, {
  jsonSchema: void 0
});
const UndefinedJsonSchema = exports.Undefined = /*#__PURE__*/Object_assign(_schemaCore.t.undefined, {
  jsonSchema: void 0
});
const VoidJsonSchema = exports.Void = /*#__PURE__*/Object_assign(_schemaCore.t.void, {
  jsonSchema: void 0
});
const BigIntJsonSchema = exports.BigInt = /*#__PURE__*/Object_assign(_schemaCore.t.bigint, {
  jsonSchema: void 0
});
const NeverJsonSchema = exports.Never = /*#__PURE__*/Object_assign(_schemaCore.t.never, {
  jsonSchema: void 0
});
const InlineJsonSchema = guard => Object_assign(_schemaCore.t.inline(guard), {
  jsonSchema: void 0
});
exports.Inline = InlineJsonSchema;
const NullJsonSchema = exports.Null = /*#__PURE__*/Object_assign(_schemaCore.t.null, {
  jsonSchema: _specification.JsonSchema.raw.null
});
const BooleanJsonSchema = exports.Boolean = /*#__PURE__*/Object_assign(_schemaCore.t.boolean, {
  jsonSchema: _specification.JsonSchema.raw.boolean
});
const IntegerJsonSchema = exports.Integer = /*#__PURE__*/Object_assign(_schemaCore.t.integer, {
  jsonSchema: _specification.JsonSchema.raw.integer
});
const NumberJsonSchema = exports.Number = /*#__PURE__*/Object_assign(_schemaCore.t.number, {
  jsonSchema: _specification.JsonSchema.raw.number
});
const StringJsonSchema = exports.String = /*#__PURE__*/Object_assign(_schemaCore.t.string, {
  jsonSchema: _specification.JsonSchema.raw.string
});
const AnyJsonSchema = exports.Any = /*#__PURE__*/Object_assign(_schemaCore.t.any, {
  jsonSchema: _specification.JsonSchema.raw.any
});
const UnknownJsonSchema = exports.Unknown = /*#__PURE__*/Object_assign(_schemaCore.t.unknown, {
  jsonSchema: _specification.JsonSchema.raw.any
});
function EqJsonSchema(value) {
  return Object_assign(_schemaCore.t.eq.def(value), {
    jsonSchema: {
      const: value
    }
  });
}
function OptionalJsonSchema(schema) {
  const optional = (schema.jsonSchema[_registry.symbol.optional] ?? 0) + 1;
  const out = Object_assign(_schemaCore.t.optional.def(schema), {
    jsonSchema: {
      ...schema.jsonSchema,
      [_registry.symbol.optional]: optional
    }
  });
  return out;
}
function ArrayJsonSchema(schema) {
  return Object_assign(_schemaCore.t.array.def(schema), {
    jsonSchema: {
      type: 'array',
      items: schema.jsonSchema
    }
  });
}
function RecordJsonSchema(schema) {
  return Object_assign(_schemaCore.t.record.def(schema), {
    jsonSchema: {
      type: 'object',
      additionalProperties: schema.jsonSchema
    }
  });
}
function UnionJsonSchema(...schemas) {
  return Object_assign(_schemaCore.t.union.def(schemas), {
    jsonSchema: {
      anyOf: schemas.map(s => s.jsonSchema)
    }
  });
}
function IntersectJsonSchema(...schemas) {
  return Object_assign(_schemaCore.t.intersect.def(schemas), {
    jsonSchema: {
      allOf: schemas.map(s => s.jsonSchema)
    }
  });
}
function TupleJsonSchema(...schemas) {
  return Object_assign(_schemaCore.t.tuple.def(schemas), {
    jsonSchema: {
      type: 'array',
      items: schemas.map(s => s.jsonSchema),
      minItems: schemas.length,
      maxItems: schemas.length,
      additionalItems: false
    }
  });
}
const isRequired = v => k => {
  const jsonSchema = v[k].jsonSchema;
  if (!(_registry.symbol.optional in v[k].jsonSchema)) return true;else {
    const optional = jsonSchema[_registry.symbol.optional];
    if (typeof optional !== 'number') return true;else return optional === 0;
  }
};
function ObjectJsonSchema(schemas) {
  const required = Object_keys(schemas).filter(isRequired(schemas));
  return Object_assign(_schemaCore.t.object.def(schemas), {
    jsonSchema: {
      type: 'object',
      required,
      properties: _registry.fn.map(schemas, (v, k) => {
        if (!required.includes(String(k))) {
          const optional = v.jsonSchema[_registry.symbol.optional];
          if (optional === 1) {
            delete v.jsonSchema[_registry.symbol.optional];
            return v.jsonSchema;
          } else {
            ;
            v.jsonSchema[_registry.symbol.optional]--;
            return v.jsonSchema;
          }
        }
        return v.jsonSchema;
      })
    }
  });
}
//# sourceMappingURL=jsonSchema.js.map