"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.any = void 0;
exports.array = array;
exports.boolean = exports.bigint = void 0;
exports.eq = eq;
exports.integer = void 0;
exports.intersect = intersect;
Object.defineProperty(exports, "minItems", {
  enumerable: true,
  get: function () {
    return _items.minItems;
  }
});
exports.number = exports.null = exports.never = void 0;
exports.object = object_;
exports.optional = optional;
exports.record = record;
exports.symbol = exports.string = void 0;
exports.tuple = tuple;
exports.undefined = void 0;
exports.union = union;
exports.void = exports.unknown = void 0;
var _registry = /*#__PURE__*/require("@traversable/registry");
var _properties = /*#__PURE__*/require("./properties.js");
var _specification = /*#__PURE__*/require("./specification.js");
var _items = /*#__PURE__*/require("./items.js");
/** @internal */
const Object_keys = globalThis.Object.keys;
const hasSchema = /*#__PURE__*/(0, _registry.has)('jsonSchema');
const schemaOf = x => (0, _registry.has)('jsonSchema')(x) ? x.jsonSchema : x;
const never_ = exports.never = {
  jsonSchema: void 0
};
const void_ = exports.void = {
  jsonSchema: void 0
};
const undefined_ = exports.undefined = {
  jsonSchema: void 0
};
const symbol_ = exports.symbol = {
  jsonSchema: void 0
};
const bigint_ = exports.bigint = {
  jsonSchema: void 0
};
/* * * * * * * * *
 *                *
 *   data types   *
 *                *
  * * * * * * * * */
const any_ = exports.any = {
  jsonSchema: _specification.JsonSchema.raw.any
};
const unknown_ = exports.unknown = {
  jsonSchema: _specification.JsonSchema.raw.any
};
const null_ = exports.null = {
  jsonSchema: _specification.JsonSchema.raw.null
};
const boolean_ = exports.boolean = {
  jsonSchema: _specification.JsonSchema.raw.boolean
};
const integer_ = exports.integer = {
  jsonSchema: _specification.JsonSchema.raw.integer
};
const number_ = exports.number = {
  jsonSchema: _specification.JsonSchema.raw.number
};
const string_ = exports.string = {
  jsonSchema: _specification.JsonSchema.raw.string
};
function eq(value) {
  return {
    jsonSchema: {
      const: value
    }
  };
}
function optional(x) {
  return {
    jsonSchema: {
      ...(hasSchema(x) && x),
      [_registry.symbol.optional]: (0, _properties.wrapOptional)(x)
    }
  };
}
function array(x) {
  return {
    jsonSchema: {
      type: 'array',
      items: schemaOf(x)
    }
  };
}
function record(x) {
  return {
    jsonSchema: {
      type: 'object',
      additionalProperties: schemaOf(x)
    }
  };
}
function union(xs) {
  return {
    jsonSchema: {
      anyOf: xs.map(schemaOf)
    }
  };
}
function intersect(xs) {
  return {
    jsonSchema: {
      allOf: xs.map(schemaOf)
    }
  };
}
function tuple(xs) {
  return {
    jsonSchema: {
      type: 'array',
      items: xs.map(schemaOf),
      minItems: (0, _items.minItems)(xs),
      maxItems: xs.length,
      additionalItems: false
    }
  };
}
function object_(xs) {
  const required = Object_keys(xs).filter((0, _properties.isRequired)(xs));
  return {
    jsonSchema: {
      type: 'object',
      required,
      properties: _registry.fn.map(xs, (0, _properties.property)(required))
    }
  };
}
//# sourceMappingURL=jsonSchema_.js.map