import type { Guard } from '@traversable/schema-core';
import { t } from '@traversable/schema-core';
import type { Mut } from '@traversable/registry';
import { symbol } from '@traversable/registry';
import { JsonSchema } from './specification.js';
export { SymbolJsonSchema as Symbol, VoidJsonSchema as Void, UndefinedJsonSchema as Undefined, BigIntJsonSchema as BigInt, NeverJsonSchema as Never, InlineJsonSchema as Inline, UnknownJsonSchema as Unknown, AnyJsonSchema as Any, NullJsonSchema as Null, BooleanJsonSchema as Boolean, IntegerJsonSchema as Integer, NumberJsonSchema as Number, StringJsonSchema as String, EqJsonSchema as Eq, OptionalJsonSchema as Optional, ArrayJsonSchema as Array, RecordJsonSchema as Record, UnionJsonSchema as Union, IntersectJsonSchema as Intersect, TupleJsonSchema as Tuple, ObjectJsonSchema as Object, };
interface SymbolJsonSchema extends t.symbol {
    jsonSchema: void;
}
declare const SymbolJsonSchema: SymbolJsonSchema;
interface UndefinedJsonSchema extends t.undefined {
    jsonSchema: void;
}
declare const UndefinedJsonSchema: UndefinedJsonSchema;
interface VoidJsonSchema extends t.void {
    jsonSchema: void;
}
declare const VoidJsonSchema: VoidJsonSchema;
interface BigIntJsonSchema extends t.bigint {
    jsonSchema: void;
}
declare const BigIntJsonSchema: BigIntJsonSchema;
interface NeverJsonSchema extends t.never {
    jsonSchema: void;
}
declare const NeverJsonSchema: NeverJsonSchema;
interface InlineJsonSchema<S> extends t.inline<S> {
    jsonSchema: void;
}
declare const InlineJsonSchema: <S>(guard: Guard<S>) => InlineJsonSchema<S>;
interface NullJsonSchema extends t.null {
    jsonSchema: Evaluate<JsonSchema.null>;
}
declare const NullJsonSchema: NullJsonSchema;
interface BooleanJsonSchema extends t.boolean {
    jsonSchema: Evaluate<JsonSchema.boolean>;
}
declare const BooleanJsonSchema: BooleanJsonSchema;
interface IntegerJsonSchema extends t.integer {
    jsonSchema: Evaluate<JsonSchema.integer>;
}
declare const IntegerJsonSchema: IntegerJsonSchema;
interface NumberJsonSchema extends t.number {
    jsonSchema: Evaluate<JsonSchema.number>;
}
declare const NumberJsonSchema: NumberJsonSchema;
interface StringJsonSchema extends t.string {
    jsonSchema: Evaluate<JsonSchema.string>;
}
declare const StringJsonSchema: StringJsonSchema;
interface AnyJsonSchema extends t.any {
    jsonSchema: Evaluate<JsonSchema.any>;
}
declare const AnyJsonSchema: AnyJsonSchema;
interface UnknownJsonSchema extends t.unknown {
    jsonSchema: Evaluate<JsonSchema.any>;
}
declare const UnknownJsonSchema: UnknownJsonSchema;
interface EqJsonSchema<S> extends t.eq.def<S> {
    jsonSchema: Evaluate<JsonSchema.const<S>>;
}
declare function EqJsonSchema<V extends Mut<V>>(value: V): EqJsonSchema<V>;
declare function EqJsonSchema<V>(value: V): EqJsonSchema<V>;
interface OptionalJsonSchema<S> extends t.optional.def<S> {
    jsonSchema: S['jsonSchema' & keyof S] & {
        [symbol.optional]: number;
    };
}
declare function OptionalJsonSchema<S>(schema: S): OptionalJsonSchema<S>;
declare function OptionalJsonSchema<S>(schema: S): OptionalJsonSchema<S>;
interface ArrayJsonSchema<S> extends t.array.def<S> {
    jsonSchema: Evaluate<JsonSchema.array<S['jsonSchema' & keyof S]>>;
}
declare function ArrayJsonSchema<S extends t.Schema>(schema: S & {
    jsonSchema: JsonSchema.Fixpoint;
}): ArrayJsonSchema<S>;
declare function ArrayJsonSchema<S>(schema: S): ArrayJsonSchema<S>;
interface RecordJsonSchema<S> extends t.record.def<S> {
    jsonSchema: Evaluate<JsonSchema.record<S['jsonSchema' & keyof S]>>;
}
declare function RecordJsonSchema<S extends t.Schema>(schema: S): RecordJsonSchema<S>;
declare function RecordJsonSchema<S>(schema: S): RecordJsonSchema<S>;
interface UnionJsonSchema<S extends readonly unknown[]> extends t.union.def<S> {
    jsonSchema: {
        anyOf: {
            [I in keyof S]: S[I]['jsonSchema' & keyof S[I]];
        };
    };
}
declare function UnionJsonSchema<S extends readonly t.Schema[]>(...schemas: S): UnionJsonSchema<S>;
declare function UnionJsonSchema<S extends readonly unknown[]>(...schemas: S): UnionJsonSchema<S>;
interface IntersectJsonSchema<S extends readonly unknown[]> extends t.intersect.def<S> {
    jsonSchema: {
        allOf: {
            [I in keyof S]: S[I]['jsonSchema' & keyof S[I]];
        };
    };
}
declare function IntersectJsonSchema<S extends readonly t.Schema[]>(...schemas: S): IntersectJsonSchema<S>;
declare function IntersectJsonSchema<S extends readonly unknown[]>(...schemas: S): IntersectJsonSchema<S>;
interface TupleJsonSchema<S extends readonly unknown[]> extends t.tuple<S> {
    jsonSchema: {
        type: 'array';
        items: {
            [I in keyof S]: S[I]['jsonSchema' & keyof S[I]];
        };
        additionalItems: false;
        minItems: S['length' & keyof S];
        maxItems: S['length' & keyof S];
    };
}
declare function TupleJsonSchema<S extends readonly t.Schema[]>(...schemas: S): TupleJsonSchema<S>;
declare function TupleJsonSchema<S extends readonly unknown[]>(...schemas: S): TupleJsonSchema<S>;
interface ObjectJsonSchema<S extends {
    [x: string]: unknown;
}> extends t.object<S> {
    jsonSchema: {
        type: 'object';
        required: Optionals<S>;
        properties: {
            [K in keyof S]: S[K]['jsonSchema' & keyof S[K]];
        };
    };
}
declare function ObjectJsonSchema<S extends {
    [x: string]: t.Fixpoint;
}>(schema: S): ObjectJsonSchema<S>;
declare function ObjectJsonSchema<S extends {
    [x: string]: JsonSchema.Fixpoint;
}>(schema: S): ObjectJsonSchema<S>;
declare function ObjectJsonSchema<S extends {
    [x: string]: unknown;
}>(schemas: S): ObjectJsonSchema<S>;
//# sourceMappingURL=jsonSchema.d.ts.map