import { fn, URI, symbol } from '@traversable/registry';
import { t } from '@traversable/schema-core';
import { JsonSchema } from './specification.js';
import { Functor } from './functor.js';
import * as to from './jsonSchema.js';
/** @internal */
const phantom = () => void 0;
/** @internal */
function handleOptionality(x) {
  let {
    [symbol.optional]: ix,
    ...y
  } = x;
  y = Recursive.fromJsonSchema(y);
  if (typeof ix !== 'number') return Recursive.fromJsonSchema(y);else while (ix-- > 0) y = t.optional(y);
  return y;
}
var Recursive;
(function (Recursive) {
  Recursive.toJsonSchema = x => {
    switch (true) {
      default:
        return fn.exhaustive(x);
      case x.tag === URI.symbol:
        return phantom();
      case x.tag === URI.bigint:
        return phantom();
      case x.tag === URI.never:
        return phantom();
      case x.tag === URI.undefined:
        return phantom();
      case x.tag === URI.void:
        return phantom();
      case x.tag === URI.any:
        return JsonSchema.raw.any;
      case x.tag === URI.unknown:
        return JsonSchema.raw.any;
      case x.tag === URI.null:
        return JsonSchema.raw.null;
      case x.tag === URI.boolean:
        return JsonSchema.raw.boolean;
      case x.tag === URI.integer:
        return JsonSchema.raw.integer;
      case x.tag === URI.number:
        return JsonSchema.raw.number;
      case x.tag === URI.string:
        return JsonSchema.raw.string;
      case x.tag === URI.optional:
        return to.Optional({
          jsonSchema: x.def
        }).jsonSchema;
      case x.tag === URI.eq:
        return to.Eq(x.def).jsonSchema;
      case x.tag === URI.array:
        return to.Array({
          jsonSchema: x.def
        }).jsonSchema;
      case x.tag === URI.record:
        return to.Record({
          ...x,
          jsonSchema: x.def
        }).jsonSchema;
      case x.tag === URI.union:
        return to.Union(...fn.map(x.def, v => ({
          jsonSchema: v
        }))).jsonSchema;
      case x.tag === URI.intersect:
        return to.Intersect(...fn.map(x.def, v => ({
          jsonSchema: v
        }))).jsonSchema;
      case x.tag === URI.tuple:
        return to.Tuple(...fn.map(x.def, v => ({
          jsonSchema: v
        }))).jsonSchema;
      case x.tag === URI.object:
        return to.Object(fn.map(x.def, v => ({
          jsonSchema: v
        }))).jsonSchema;
    }
  };
  Recursive.fromJsonSchema = x => {
    switch (true) {
      default:
        return fn.exhaustive(x);
      case symbol.optional in x:
        return handleOptionality(x);
      case JsonSchema.null(x):
        return t.null;
      case JsonSchema.boolean(x):
        return t.boolean;
      case JsonSchema.integer(x):
        return t.integer;
      case JsonSchema.number(x):
        return t.number;
      case JsonSchema.string(x):
        return t.string;
      case JsonSchema.record(x):
        return t.record(x.additionalProperties);
      case JsonSchema.any(x):
        return t.unknown;
      case JsonSchema.union(x):
        return t.union.def(x.anyOf);
      case JsonSchema.enum(x):
        return t.union.def(x.enum.map(_ => t.eq(_)));
      case JsonSchema.const(x):
        return t.eq(x.const);
      case JsonSchema.intersect(x):
        return t.intersect.def(x.allOf);
      case JsonSchema.tuple(x):
        return t.tuple.def(x.items);
      case JsonSchema.array(x):
        return t.array.def(x.items);
      case JsonSchema.record(x):
        return t.record.def(x.additionalProperties);
      case JsonSchema.object(x):
        return t.object.def(fn.map(x.properties, (v, k) => x.required.includes(`${k}`) ? v : t.optional(v)));
    }
  };
})(Recursive || (Recursive = {}));
export const toJsonSchema = /*#__PURE__*/fn.cata(t.Functor)(Recursive.toJsonSchema);
export const fromJsonSchema = /*#__PURE__*/fn.cata(Functor)(Recursive.fromJsonSchema);
//# sourceMappingURL=adapter.js.map