import { t } from '@traversable/schema-core';
import { fn, symbol } from '@traversable/registry';
import { JsonSchema } from './specification.js';
export { /* schemas that don't have a corresponding JSON Schema representation */
SymbolJsonSchema as Symbol, VoidJsonSchema as Void, UndefinedJsonSchema as Undefined, BigIntJsonSchema as BigInt, NeverJsonSchema as Never, InlineJsonSchema as Inline, /* data types */
UnknownJsonSchema as Unknown, AnyJsonSchema as Any, NullJsonSchema as Null, BooleanJsonSchema as Boolean, IntegerJsonSchema as Integer, NumberJsonSchema as Number, StringJsonSchema as String, EqJsonSchema as Eq, OptionalJsonSchema as Optional, ArrayJsonSchema as Array, RecordJsonSchema as Record, UnionJsonSchema as Union, IntersectJsonSchema as Intersect, TupleJsonSchema as Tuple, ObjectJsonSchema as Object };
/** @internal */
const Object_assign = globalThis.Object.assign;
/** @internal */
const Object_keys = globalThis.Object.keys;
const SymbolJsonSchema = /*#__PURE__*/Object_assign(t.symbol, {
  jsonSchema: void 0
});
const UndefinedJsonSchema = /*#__PURE__*/Object_assign(t.undefined, {
  jsonSchema: void 0
});
const VoidJsonSchema = /*#__PURE__*/Object_assign(t.void, {
  jsonSchema: void 0
});
const BigIntJsonSchema = /*#__PURE__*/Object_assign(t.bigint, {
  jsonSchema: void 0
});
const NeverJsonSchema = /*#__PURE__*/Object_assign(t.never, {
  jsonSchema: void 0
});
const InlineJsonSchema = guard => Object_assign(t.inline(guard), {
  jsonSchema: void 0
});
const NullJsonSchema = /*#__PURE__*/Object_assign(t.null, {
  jsonSchema: JsonSchema.raw.null
});
const BooleanJsonSchema = /*#__PURE__*/Object_assign(t.boolean, {
  jsonSchema: JsonSchema.raw.boolean
});
const IntegerJsonSchema = /*#__PURE__*/Object_assign(t.integer, {
  jsonSchema: JsonSchema.raw.integer
});
const NumberJsonSchema = /*#__PURE__*/Object_assign(t.number, {
  jsonSchema: JsonSchema.raw.number
});
const StringJsonSchema = /*#__PURE__*/Object_assign(t.string, {
  jsonSchema: JsonSchema.raw.string
});
const AnyJsonSchema = /*#__PURE__*/Object_assign(t.any, {
  jsonSchema: JsonSchema.raw.any
});
const UnknownJsonSchema = /*#__PURE__*/Object_assign(t.unknown, {
  jsonSchema: JsonSchema.raw.any
});
function EqJsonSchema(value) {
  return Object_assign(t.eq.def(value), {
    jsonSchema: {
      const: value
    }
  });
}
function OptionalJsonSchema(schema) {
  const optional = (schema.jsonSchema[symbol.optional] ?? 0) + 1;
  const out = Object_assign(t.optional.def(schema), {
    jsonSchema: {
      ...schema.jsonSchema,
      [symbol.optional]: optional
    }
  });
  return out;
}
function ArrayJsonSchema(schema) {
  return Object_assign(t.array.def(schema), {
    jsonSchema: {
      type: 'array',
      items: schema.jsonSchema
    }
  });
}
function RecordJsonSchema(schema) {
  return Object_assign(t.record.def(schema), {
    jsonSchema: {
      type: 'object',
      additionalProperties: schema.jsonSchema
    }
  });
}
function UnionJsonSchema(...schemas) {
  return Object_assign(t.union.def(schemas), {
    jsonSchema: {
      anyOf: schemas.map(s => s.jsonSchema)
    }
  });
}
function IntersectJsonSchema(...schemas) {
  return Object_assign(t.intersect.def(schemas), {
    jsonSchema: {
      allOf: schemas.map(s => s.jsonSchema)
    }
  });
}
function TupleJsonSchema(...schemas) {
  return Object_assign(t.tuple.def(schemas), {
    jsonSchema: {
      type: 'array',
      items: schemas.map(s => s.jsonSchema),
      minItems: schemas.length,
      maxItems: schemas.length,
      additionalItems: false
    }
  });
}
const isRequired = v => k => {
  const jsonSchema = v[k].jsonSchema;
  if (!(symbol.optional in v[k].jsonSchema)) return true;else {
    const optional = jsonSchema[symbol.optional];
    if (typeof optional !== 'number') return true;else return optional === 0;
  }
};
function ObjectJsonSchema(schemas) {
  const required = Object_keys(schemas).filter(isRequired(schemas));
  return Object_assign(t.object.def(schemas), {
    jsonSchema: {
      type: 'object',
      required,
      properties: fn.map(schemas, (v, k) => {
        if (!required.includes(String(k))) {
          const optional = v.jsonSchema[symbol.optional];
          if (optional === 1) {
            delete v.jsonSchema[symbol.optional];
            return v.jsonSchema;
          } else {
            ;
            v.jsonSchema[symbol.optional]--;
            return v.jsonSchema;
          }
        }
        return v.jsonSchema;
      })
    }
  });
}
//# sourceMappingURL=jsonSchema.js.map