import { fn, has, symbol } from '@traversable/registry';
import { isRequired, wrapOptional, property } from './properties.js';
import { JsonSchema } from './specification.js';
import { minItems } from './items.js';
export { minItems } from './items.js';
export { never_ as never, unknown_ as unknown, any_ as any, void_ as void, undefined_ as undefined, null_ as null, symbol_ as symbol, boolean_ as boolean, bigint_ as bigint, integer_ as integer, number_ as number, string_ as string, eq, optional, array, record, tuple, union, intersect, object_ as object };
/** @internal */
const Object_keys = globalThis.Object.keys;
const hasSchema = /*#__PURE__*/has('jsonSchema');
const schemaOf = x => has('jsonSchema')(x) ? x.jsonSchema : x;
const never_ = {
  jsonSchema: void 0
};
const void_ = {
  jsonSchema: void 0
};
const undefined_ = {
  jsonSchema: void 0
};
const symbol_ = {
  jsonSchema: void 0
};
const bigint_ = {
  jsonSchema: void 0
};
/* * * * * * * * *
 *                *
 *   data types   *
 *                *
  * * * * * * * * */
const any_ = {
  jsonSchema: JsonSchema.raw.any
};
const unknown_ = {
  jsonSchema: JsonSchema.raw.any
};
const null_ = {
  jsonSchema: JsonSchema.raw.null
};
const boolean_ = {
  jsonSchema: JsonSchema.raw.boolean
};
const integer_ = {
  jsonSchema: JsonSchema.raw.integer
};
const number_ = {
  jsonSchema: JsonSchema.raw.number
};
const string_ = {
  jsonSchema: JsonSchema.raw.string
};
function eq(value) {
  return {
    jsonSchema: {
      const: value
    }
  };
}
function optional(x) {
  return {
    jsonSchema: {
      ...(hasSchema(x) && x),
      [symbol.optional]: wrapOptional(x)
    }
  };
}
function array(x) {
  return {
    jsonSchema: {
      type: 'array',
      items: schemaOf(x)
    }
  };
}
function record(x) {
  return {
    jsonSchema: {
      type: 'object',
      additionalProperties: schemaOf(x)
    }
  };
}
function union(xs) {
  return {
    jsonSchema: {
      anyOf: xs.map(schemaOf)
    }
  };
}
function intersect(xs) {
  return {
    jsonSchema: {
      allOf: xs.map(schemaOf)
    }
  };
}
function tuple(xs) {
  return {
    jsonSchema: {
      type: 'array',
      items: xs.map(schemaOf),
      minItems: minItems(xs),
      maxItems: xs.length,
      additionalItems: false
    }
  };
}
function object_(xs) {
  const required = Object_keys(xs).filter(isRequired(xs));
  return {
    jsonSchema: {
      type: 'object',
      required,
      properties: fn.map(xs, property(required))
    }
  };
}
//# sourceMappingURL=jsonSchema_.js.map