import { t } from '@traversable/schema-core';
const RAW = {
  any: {
    type: 'object',
    properties: {},
    nullable: true
  },
  null: {
    type: 'null',
    enum: [null]
  },
  boolean: {
    type: 'boolean'
  },
  integer: {
    type: 'integer'
  },
  number: {
    type: 'number'
  },
  string: {
    type: 'string'
  }
};
const JsonSchema_any = /*#__PURE__*/t.object({
  type: /*#__PURE__*/t.eq('object'),
  properties: /*#__PURE__*/t.object({}),
  nullable: /*#__PURE__*/t.eq(true)
});
const JsonSchema_null = /*#__PURE__*/t.object({
  type: /*#__PURE__*/t.eq('null'),
  enum: /*#__PURE__*/t.tuple(/*#__PURE__*/t.eq(null))
});
const JsonSchema_boolean = /*#__PURE__*/t.object({
  type: /*#__PURE__*/t.eq('boolean')
});
const JsonSchema_integer = /*#__PURE__*/t.object({
  type: /*#__PURE__*/t.eq('integer')
});
const JsonSchema_number = /*#__PURE__*/t.object({
  type: /*#__PURE__*/t.eq('number')
});
const JsonSchema_string = /*#__PURE__*/t.object({
  type: /*#__PURE__*/t.eq('string')
});
const JsonSchema_const = /*#__PURE__*/t.object({
  const: t.unknown
});
const JsonSchema_enum = /*#__PURE__*/t.object({
  enum: /*#__PURE__*/t.array(t.unknown, 'readonly')
});
const JsonSchema_array = /*#__PURE__*/t.object({
  type: /*#__PURE__*/t.eq('array'),
  items: t.unknown
});
const JsonSchema_record_ = /*#__PURE__*/t.object({
  type: /*#__PURE__*/t.eq('object'),
  additionalProperties: t.unknown
});
const JsonSchema_record = u => JsonSchema_record_(u);
const JsonSchema_union_ = /*#__PURE__*/t.object({
  anyOf: /*#__PURE__*/t.array(t.unknown, 'readonly')
});
const JsonSchema_union = /*#__PURE__*/t.inline(u => JsonSchema_union_(u));
const JsonSchema_intersect_ = /*#__PURE__*/t.object({
  allOf: /*#__PURE__*/t.array(t.unknown, 'readonly')
});
const JsonSchema_intersect = /*#__PURE__*/t.inline(u => JsonSchema_intersect_(u));
const JsonSchema_object = /*#__PURE__*/t.object({
  type: /*#__PURE__*/t.eq('object'),
  required: /*#__PURE__*/t.array(t.string),
  properties: t.unknown
});
const JsonSchema_tuple = /*#__PURE__*/t.object({
  type: /*#__PURE__*/t.eq('array'),
  items: t.unknown,
  minItems: t.number,
  maxItems: t.number,
  additionalItems: /*#__PURE__*/t.eq(false)
});
const Nullary = /*#__PURE__*/t.union(JsonSchema_any, JsonSchema_null, JsonSchema_boolean, JsonSchema_integer, JsonSchema_number, JsonSchema_string);
const Special = /*#__PURE__*/t.union(JsonSchema_enum, JsonSchema_const);
const Unary = /*#__PURE__*/t.union(JsonSchema_array, JsonSchema_record, JsonSchema_union, JsonSchema_intersect, JsonSchema_tuple, JsonSchema_object);
export var JsonSchema;
(function (JsonSchema) {
  JsonSchema.raw = RAW;
})(JsonSchema || (JsonSchema = {}));
export const any = JsonSchema_any;
/* data types */
JsonSchema.null = JsonSchema_null;
JsonSchema.boolean = JsonSchema_boolean;
JsonSchema.integer = JsonSchema_integer;
JsonSchema.number = JsonSchema_number;
JsonSchema.string = JsonSchema_string;
JsonSchema.any = JsonSchema_any;
JsonSchema.enum = JsonSchema_enum;
JsonSchema.const = JsonSchema_const;
JsonSchema.array = JsonSchema_array;
JsonSchema.record = JsonSchema_record;
JsonSchema.union = JsonSchema_union;
JsonSchema.intersect = JsonSchema_intersect;
JsonSchema.tuple = JsonSchema_tuple;
JsonSchema.object = JsonSchema_object;
/* terms */
JsonSchema.Nullary = Nullary;
JsonSchema.Unary = Unary;
JsonSchema.Special = Special;
//# sourceMappingURL=specification.js.map